<?php
class Model_ScanModule extends Model_esia {
	const DATA_PATH='/usr/local/esia/public_html/data/scan';
	const PENDING_TASK='pending';
	const PROCESSING_TASK='processing';
	const TERMINATED_TASK='terminated';
	const ERROR_TASK='error';
	
	private static function rowToArray($row){
		if(empty($row)) return null;
		$task['id'] 			= $row->scan_id;
		$task['type']			= $row->scan_type;
		$task['changed_time']	= $row->scan_changed_status_time;
		$task['actual_status']	= $row->scan_actual_status;
		$task['start_addr']		= $row->scan_start_addr;
		$task['end_addr']		= $row->scan_end_addr;
		$task['options']		= $row->scan_options;
		$task['scan_agent_id']	= $row->scan_agent_id;
		$task['data_path']		= $row->scan_data_path;
		$task['error_message']	= $row->scan_error_message;
		return $task;
	}
	
	public static function startScanningTask($dbconnection,$start_addr,$end_addr,$scan_agent_id=null,$type='ICMP',$options=null){
		$time = time();
		$status = Model_ScanModule::PENDING_TASK;

		$sql = "INSERT INTO esia_scan(scan_changed_status_time,scan_actual_status,scan_start_addr,scan_end_addr,scan_agent_id,scan_type,scan_options)
				VALUES (current_timestamp,'$status',:start,:end,:agent,:type,:options) RETURNING scan_id;";
		
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array(
				':start'=>$start_addr,
				':end'=>$end_addr,
				':agent'=>$scan_agent_id,
				':type'=>$type,
				':options'=>$options
		));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->scan_id;
	}
	
	public static function getScanningTask($dbconnection,$task_id){
		$sql = 'SELECT * FROM esia_scan WHERE scan_id = :id';
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array(':id'=>$task_id));
		
		$row=$query->fetch(PDO::FETCH_OBJ);	
		return Model_ScanModule::rowToArray($row);
	}
	
	public static function getScanningTaskList($dbconnection,$all_fields=false,$status=null){
		if($all_fields) $select = '*';
		else 			$select = 'scan_id';
		
		if($status == null) 	{
			$sql = "SELECT $select from esia_scan";
			$params = array();
		}
		else {
			$sql = "SELECT $select from esia_scan WHERE scan_actual_status=:status";
			$params = array('status'=>$status);
		}
					
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,$params);
		
		$list=array();
		if($all_fields){
			while($row=$query->fetch(PDO::FETCH_OBJ)){
				$list[]=Model_ScanModule::rowToArray($row);
			}
		}
		else{
			while($row=$query->fetch(PDO::FETCH_OBJ)){
				$list[]=$row->scan_id;
			}
		} 
		return $list;
	}
	
	public static function getGroupedByDevicesTaskIds($dbconnection,$status=null){
		if($status == null) 	{
			$sql = 'SELECT scan_id,scan_agent_id from esia_scan';
			$params = array();
		}
		else {
			$sql = 'SELECT scan_id,scan_agent_id from esia_scan WHERE scan_actual_status=:status';
			$params = array('status'=>$status);
		}
			
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,$params);
		
		$list = array();
		while($row=$query->fetch(PDO::FETCH_OBJ)){
			$agent_id = 'this';
			if($row->scan_agent_id != null) $agent_id =  $row->scan_agent_id;
			
			if(empty($list[$agent_id]))	$list[$agent_id] = array();
			array_push($list[$agent_id],$row->scan_id);
		}
		return $list;
	}
	
	public static function updateScanningTaskStatus($dbconnection,$task_id,$status,$data_path=null){	
		if($data_path == null) {
			$set = 'scan_changed_status_time=current_timestamp,scan_actual_status=:new_status';
			$params = array(':new_status'=>$status);
		}
		else{
			$set = 'scan_changed_status_time=current_timestamp,scan_actual_status=:new_status,scan_data_path=:data_path';
			$params = array(':new_status'=>$status,':data_path'=>$data_path);
		} 
			
		$sql = "UPDATE esia_scan SET $set WHERE scan_id=:task_id";
		$query = $dbconnection->prepare($sql);
		
		if(is_int($task_id)) $task_id=array($task_id);
		foreach($task_id as $id){
			$params[':task_id'] = $id;
			$query = parent::executeStatement($query,$params);
		}
	}
	
	public static function notifyScanningError($dbconnection,$task_id,$message){
		$sql = "UPDATE esia_scan SET 
				scan_changed_status_time=current_timestamp,
				scan_actual_status='".Model_ScanModule::ERROR_TASK."',
				scan_error_message=:message 
				WHERE scan_id=:task_id";
		$query = $dbconnection->prepare($sql);
		
		if(is_int($task_id)) $task_id=array($task_id);
		foreach($task_id as $id){
			$query = parent::executeStatement($query,array(':message'=>$message,':task_id'=>$id));
		}
	}
	
	public static function deleteScanningTask($dbconnection,$task_id){
		$sql = 'DELETE FROM esia_scan WHERE scan_id=:task_id';
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array(':task_id'=>$task_id));
	}
}