<?php
define('LOG_FILE','/var/log/esia/scan_collector.log');
define('LOG_LEVEL',LOG_DEBUG);

function write_log($priority,$message){
	if($priority <= LOG_LEVEL){
		$time = sprintf("%.4f",microtime(true));
		file_put_contents ( LOG_FILE , "$time|$priority|$message\n" ,  FILE_APPEND  );
	}
	if( $priority == LOG_EMERG) exit(1);
}

$_SERVER['DOCUMENT_ROOT'] = '/usr/local/esia/public_html/';

$path = $_SERVER['DOCUMENT_ROOT'];
include $path.'/esia.conf.php';
include_once USR_CLASS;
include_once LIB_LOG;
include_once MVCMODEL;
include_once $path.'/Model_esia.php';

write_log(LOG_NOTICE,'START scan collector');

//connection à la base de donnée
write_log(LOG_INFO,'DB connection');
try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	$err='Unable to connect to DB. ';
	$err.='PDO SQL ERROR N° : '.$e->getCode();
	$err.='  MESSAGE : '.$e->getMessage();
	write_log(LOG_EMERG,$err);
}

$arr = Model_esia::getWebPlugin($dbconnection,false,'WEBP_IMPORT');
$plugin_url = $arr[0]['apache_path'];
include_once $path.$plugin_url.'/inc/Model_import-module.php';


define('PLUGINS_PATH','/usr/local/esia/plugins');
define('TMP_DIR',"/tmp/scan");
$scanner = new ScanManager();

try{
	$pending_tasks = Model_ScanModule::getScanningTaskList($dbconnection,true,Model_ScanModule::PENDING_TASK);
	$processing_tasks = Model_ScanModule::getGroupedByDevicesTaskIds($dbconnection,Model_ScanModule::PROCESSING_TASK);
}catch(Exception $e)
{
	$err='Unable get task from DB. ';
	$err.='PDO SQL ERROR N° : '.$e->getCode();
	$err.='  MESSAGE : '.$e->getMessage();
	write_log(LOG_ALERT,$err);
}

//Traitement des task pending
write_log(LOG_INFO,'Sending '.count($pending_tasks).' pending tasks');

$new_processing_task=array();
foreach($pending_tasks as $task){
	array_push($new_processing_task,$task['id']);
}

try{
	Model_ScanModule::updateScanningTaskStatus($dbconnection,$new_processing_task,Model_ScanModule::PROCESSING_TASK);
}
catch(Exception $ex){
	$err='Unable to set status processing in DB. ';
	$err.='PDO SQL ERROR N° : '.$e->getCode();
	$err.='  MESSAGE : '.$e->getMessage();
	write_log(LOG_ALERT,$err);
}

foreach($pending_tasks as $task){
	$id			= $task['id'];
	$device	  	= $task['scan_agent_id'];
	$first_addr = $task['start_addr'];
	$last_addr	= $task['end_addr'];
	$type		= $task['type'];
	$options	= $task['options'];

	write_log(LOG_DEBUG,"Sending task $id");
	$arguments = "-A -i $id -a $first_addr-$last_addr -t $type -o $options";
	$result = $scanner->execute($arguments,$device);

	if( $result['return'] != 0 ){ //Une erreur s'est produite.
		preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches);
		$details=$matches[1];
		$error_message="Unable to save the task => $details.";
		Model_ScanModule::notifyScanningError($dbconnection,$id,$error_message);
		write_log(LOG_WARNING,$error_message);
		echo $error_message."\n";
	}
}

//Récupération des task terminées.
write_log(LOG_INFO,'Get terminated tasks list for '.count($processing_tasks).' agents');

foreach($processing_tasks as $agent => $db_ids){
	write_log(LOG_INFO,"Get terminated tasks status for Agent $agent");
	if( $agent == 'this' )	$agent = null;

	$arguments = "-L -s terminated,error";
	$result = $scanner->execute($arguments,$agent);

	if( $result['return'] != 0 ){ //Une erreur s'est produite.
		preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches);
		$details=$matches[1];
		$error_message="Unable to get id list => $details.";
		write_log(LOG_CRIT,$error_message);
		echo $error_message."\n";
		continue;
	}
	else{
		$for_delete_ids = array();
		$for_get_ids = array();


		$dev_ids = trim($result['output'][0]);
		if(!empty($dev_ids)) $dev_ids = explode(',',$dev_ids);
		else $dev_ids = array();

		foreach($dev_ids as $dev_id){
			$for_delete_ids[]=$dev_id;
				
			if(in_array($dev_id,$db_ids)){
				$for_get_ids[]=$dev_id;
			}
		}
	}

	write_log(LOG_INFO,"Download ".count($for_get_ids)." result");
	if(!empty($for_get_ids)){

		$scan_ids = implode(',',$for_get_ids);
		$arguments = "-G -i $scan_ids";
		$result = $scanner->execute($arguments,$agent);

		if( $result['return'] != 0 ){ //Une erreur s'est produite.
			preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches);
			$details=$matches[1];
			$error_message="Unable to  => $details.";
			echo $error_message."\n";
			write_log(LOG_ERR,$error_message);
			continue;
		}

		$parsed = @json_decode(trim(implode("\n",$result['output'])),true);

		if($parsed == null){
			$error_message = "Unable to parse received data for $scan_ids";
			echo $error_message."\n";
			write_log(LOG_ERR,$error_message);
			continue;
		}
		else {
			foreach($parsed as $scan_id => $scan_result){
				if($scan_result['status'] == Model_ScanModule::ERROR_TASK){
					Model_ScanModule::notifyScanningError($dbconnection,$scan_id,$scan_result['message']);
					write_log(LOG_WARNING,$scan_result['message']);
				}
				else if($scan_result['status'] == Model_ScanModule::TERMINATED_TASK){
					$data_path = Model_ScanModule::DATA_PATH.'/'.$scan_id;
						
					try{
						Model_ScanModule::updateScanningTaskStatus($dbconnection,$scan_id,Model_ScanModule::TERMINATED_TASK,$data_path);
						write_log(LOG_DEBUG,"Task $scan_id setted TERMINATED in path $data_path");
					}
					catch(Exception $ex){
						Model_ScanModule::notifyScanningError($dbconnection,$scan_id,"Unable to update status for ".$scan_id);
						write_log(LOG_ERR,"Unable save result for ".$scan_id);
						continue;
					}

					write_log(LOG_DEBUG,"Save Task $scan_id in path $data_path");
					$raw = @json_encode($parsed[$scan_id],JSON_PRETTY_PRINT);
					if (!@file_put_contents($data_path,$raw)){
						Model_ScanModule::notifyScanningError($dbconnection,$scan_id,"Unable save result for ".$scan_id);
						write_log(LOG_WARNING,"Unable save result for ".$scan_id);
					}
				}
			}
		}
	}

	write_log(LOG_INFO,"Delete ".count($for_delete_ids)." terminated tasks for agent $agent");
	if(!empty($for_delete_ids)){
		$scan_ids = implode(',',$for_delete_ids);
		$arguments = "-D -i $scan_ids";
		$result = $scanner->execute($arguments,$agent);

		if( $result['return'] != 0 ){ //Une erreur s'est produite.
			preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches);
			$details=$matches[1];
			$error_message="Unable to delete scans  => $details.";
			write_log(LOG_WARNING,$error_message);
			echo $error_message."\n";
		}
	}
}
write_log(LOG_NOTICE,'END scan collector');

class ScanManager{
	private $plugin;

	public function __construct(){
		$this->plugin = new ExecPlugin(1,60);
	}

	public function execute($arguments,$gesa=null){
		$result = $this->plugin->execute('scan_task_adm.php', $arguments,$gesa);
		if( $gesa!= null && $result['return'] == 3){
			preg_match('/UNKNOWN\s*-\s*(.*)/',$result['output'][0],$matches);
			$result['output'] = array('ERROR : '.$matches[1]);
		}
		return $result;
	}
}

class ExecPlugin{
	const CHECK_GESA='check_gesa.php';

	private $_timeout;
	private $_priority;

	public function __construct($priority=3,$timeout=3){
		$this->_priority = $priority;
		$this->_timeout = $timeout;
	}

	public function execute($command,$arguments,$gesa=null){
		if( $gesa == null )exec(PLUGINS_PATH.'/'.$command.' '.$arguments,$output,$returnexec);
		else{
			$timeout = $this->_timeout;
			$pri = $this->_priority;
			exec(PLUGINS_PATH.'/'.self::CHECK_GESA." -G $gesa -C $command -A '$arguments' -t $timeout -p $pri",$output,$returnexec);
		}
		return array ("output"=>$output,"return"=>$returnexec);
	}
}
?>